@@ -34,7 +34,6 @@
 
 #include <algorithm>
 #include <string>
-#include <set>
 #include <vector>
 
 #include "publicationset.h"
@@ -45,7 +44,7 @@ PublicationSet::PublicationSet(const size_t maxSize)
 {
    maxEntries = maxSize;
    publicationArray = new Node*[maxEntries];
-   assert(publicationArray != NULL);
+   assert(publicationArray != nullptr);
    clearAll();
 }
 
@@ -64,7 +63,7 @@ void PublicationSet::clearAll()
 {
    entries = 0;
    for(size_t i = 0;i < maxEntries; i++) {
-      publicationArray[i] = NULL;
+      publicationArray[i] = nullptr;
    }
 }
 
@@ -87,7 +86,7 @@ bool PublicationSet::add(Node* publication)
 // ###### Add all nodes from collection #####################################
 void PublicationSet::addAll(Node* publication)
 {
-   while(publication != NULL) {
+   while(publication != nullptr) {
       if(add(publication)) {
          publication->anchor = publication->keyword;
       }
@@ -97,8 +96,8 @@ void PublicationSet::addAll(Node* publication)
 
 
 // NOTE: PublicationSet::sort() will *NOT* be thread-safe!
-static const std::string* gSortKey       = NULL;
-static const bool*        gSortAscending = NULL;
+static const std::string* gSortKey       = nullptr;
+static const bool*        gSortAscending = nullptr;
 static size_t             gMaxSortLevels = 0;
 
 // ###### Node comparison function for qsort() ##############################
@@ -111,13 +110,13 @@ static int publicationNodeComparisonFunction(const void* ptr1, const void* ptr2)
       const Node* child1 = findChildNode((Node*)node1, gSortKey[i].c_str());
       const Node* child2 = findChildNode((Node*)node2, gSortKey[i].c_str());
       int result = 0;
-      if( (child1 == NULL) && (child2 != NULL) ) {
+      if( (child1 == nullptr) && (child2 != nullptr) ) {
          result = 1;
       }
-      if( (child1 != NULL) && (child2 == NULL) ) {
+      if( (child1 != nullptr) && (child2 == nullptr) ) {
          result = -1;
       }
-      else if( (child1 != NULL) && (child2 != NULL) ) {
+      else if( (child1 != nullptr) && (child2 != nullptr) ) {
          if(child1->value < child2->value) {
             result = -1;
          }
@@ -148,8 +147,8 @@ void PublicationSet::sort(const std::string* sortKey,
 
    qsort(publicationArray, entries, sizeof(Node*), publicationNodeComparisonFunction);
 
-   gSortKey       = NULL;
-   gSortAscending = NULL;
+   gSortKey       = nullptr;
+   gSortAscending = nullptr;
 }
 
 
@@ -173,7 +172,7 @@ std::string PublicationSet::makeDownloadFileName(const char*        downloadDire
       extension = ".txt";
    }
 
-   if( (downloadDirectory != NULL) && (strlen(downloadDirectory) != 0) ) {
+   if( (downloadDirectory != nullptr) && (strlen(downloadDirectory) != 0) ) {
       return((std::string)downloadDirectory + "/" + anchor + extension);
    }
    return(anchor + extension);
@@ -188,10 +187,10 @@ bool PublicationSet::exportPublicationSetToBibTeX(PublicationSet* publicationSet
                                                   const bool      addNotesWithISBNandISSN,
                                                   const bool      addUrlCommand)
 {
-   FILE* fh = NULL;
+   FILE* fh = nullptr;
    if(!separateFiles) {
       fh = fopen(fileNamePrefix, "w");
-      if(fh == NULL) {
+      if(fh == nullptr) {
          fprintf(stderr, "ERROR: Unable to create BibTeX file %s!\n", fileNamePrefix);
          return(false);
       }
@@ -200,7 +199,7 @@ bool PublicationSet::exportPublicationSetToBibTeX(PublicationSet* publicationSet
    for(size_t index = 0; index < publicationSet->size(); index++) {
       const Node* publication = publicationSet->get(index);
       if(publication->value == "Comment") {
-         if(fh != NULL) {
+         if(fh != nullptr) {
             fprintf(fh, "%%%s\n\n", publication->keyword.c_str());
          }
       }
@@ -209,7 +208,7 @@ bool PublicationSet::exportPublicationSetToBibTeX(PublicationSet* publicationSet
             char fileName[1024];
             snprintf((char*)&fileName, sizeof(fileName), "%s%s.bib", fileNamePrefix, publication->keyword.c_str());
             fh = fopen(fileName, "w");
-            if(fh == NULL) {
+            if(fh == nullptr) {
                fprintf(stderr, "ERROR: Unable to create XML file %s!\n", fileName);
                return(false);
             }
@@ -220,10 +219,10 @@ bool PublicationSet::exportPublicationSetToBibTeX(PublicationSet* publicationSet
 
          bool  empty           = true;
          Node* child           = publication->child;
-         const Node* issn      = NULL;
-         const Node* isbn      = NULL;
+         const Node* issn      = nullptr;
+         const Node* isbn      = nullptr;
          const char* separator = "";
-         while(child != NULL) {
+         while(child != nullptr) {
             if(!empty) {
                separator = ",\n";
             }
@@ -282,7 +281,7 @@ bool PublicationSet::exportPublicationSetToBibTeX(PublicationSet* publicationSet
          }
 
          if( (addNotesWithISBNandISSN) &&
-             ((isbn != NULL) || (issn != NULL)) ) {
+             ((isbn != nullptr) || (issn != nullptr)) ) {
             if(isbn) {
                fprintf(fh, "%s\tnote = \"{ISBN} %s\"", separator, isbn->value.c_str());
             }
@@ -294,9 +293,9 @@ bool PublicationSet::exportPublicationSetToBibTeX(PublicationSet* publicationSet
          fputs("\n}\n\n", fh);
       }
 
-      if( (separateFiles) && (fh != NULL)) {
+      if( (separateFiles) && (fh != nullptr)) {
          fclose(fh);
-         fh = NULL;
+         fh = nullptr;
       }
    }
 
@@ -312,10 +311,10 @@ bool PublicationSet::exportPublicationSetToXML(PublicationSet* publicationSet,
                                                const char*     fileNamePrefix,
                                                const bool      separateFiles)
 {
-   FILE* fh = NULL;
+   FILE* fh = nullptr;
    if(!separateFiles) {
       fh = fopen(fileNamePrefix, "w");
-      if(fh == NULL) {
+      if(fh == nullptr) {
          fprintf(stderr, "ERROR: Unable to create XML file %s!\n", fileNamePrefix);
          return(false);
       }
@@ -327,7 +326,7 @@ bool PublicationSet::exportPublicationSetToXML(PublicationSet* publicationSet,
       Node* publication = publicationSet->get(index);
 
       if(publication->value == "Comment") {
-         if(fh != NULL) {
+         if(fh != nullptr) {
             fprintf(fh, "<!-- %s -->\n\n", publication->keyword.c_str());
          }
       }
@@ -336,7 +335,7 @@ bool PublicationSet::exportPublicationSetToXML(PublicationSet* publicationSet,
             char fileName[1024];
             snprintf((char*)&fileName, sizeof(fileName), "%s%s.xml", fileNamePrefix, publication->keyword.c_str());
             fh = fopen(fileName, "w");
-            if(fh == NULL) {
+            if(fh == nullptr) {
                fprintf(stderr, "ERROR: Unable to create XML file %s!\n", fileName);
                return(false);
             }
@@ -363,7 +362,7 @@ bool PublicationSet::exportPublicationSetToXML(PublicationSet* publicationSet,
          const Node* issn         = findChildNode(publication, "issn");
          const Node* doi          = findChildNode(publication, "doi");
 
-         if(url == NULL) {
+         if(url == nullptr) {
             fprintf(fh, "<reference anchor=\"%s\">\n",
                     labelToXMLLabel(publication->keyword).c_str());
          }
@@ -428,7 +427,7 @@ bool PublicationSet::exportPublicationSetToXML(PublicationSet* publicationSet,
                   seriesValue += ", ";
                }
                seriesValue += number->value;
-               number = NULL;
+               number = nullptr;
             }
          }
          if(number) {
@@ -505,9 +504,9 @@ bool PublicationSet::exportPublicationSetToXML(PublicationSet* publicationSet,
          fputs("</reference>\n\n", fh);
       }
 
-      if( (separateFiles) && (fh != NULL)) {
+      if( (separateFiles) && (fh != nullptr)) {
          fclose(fh);
-         fh = NULL;
+         fh = nullptr;
       }
    }
 
@@ -560,12 +559,13 @@ std::string PublicationSet::applyTemplate(Node*                           public
                                           const std::string&              lineBreak,
                                           const bool                      xmlStyle,
                                           const char*                     downloadDirectory,
+                                          const Mappings&                 mappings,
                                           FILE*                           fh)
 {
    std::string             result;
    std::vector<StackEntry> stack;
    Node*                   child;
-   Node*                   author               = NULL;
+   Node*                   author               = nullptr;
    size_t                  authorIndex          = 0;
    size_t                  authorBegin          = std::string::npos;
    bool                    skip                 = false;
@@ -578,19 +578,19 @@ std::string PublicationSet::applyTemplate(Node*                           public
    for(size_t i = 0; i < printingTemplateSize; i++) {
       if( (printingTemplate[i] == '%') && (i + 1 < printingTemplateSize) ) {
          const std::string action = getNextAction((const char*)&printingTemplate[i + 1], i);
-         if( (action == "L") || (action == "label") ) {   // Original BibTeX label
+         if(action == "label") {   // Original BibTeX label
             result += string2utf8(publication->keyword, nbsp, lineBreak, xmlStyle);
          }
          else if(action == "html-label") {   // Original BibTeX label
             result += labelToHTMLLabel(string2utf8(publication->keyword, nbsp, lineBreak, xmlStyle));
          }
-         else if( (action == "C") || (action == "anchor") ) {   // Anchor
+         else if(action == "anchor") {   // Anchor
             result += string2utf8(publication->anchor, nbsp, lineBreak, xmlStyle);
          }
-         else if( (action == "c") || (action == "class") ) {   // Class (e.g. TechReport, InProceedings, etc.)
+         else if(action == "class") {   // Class (e.g. TechReport, InProceedings, etc.)
             result += string2utf8(publication->value, nbsp, lineBreak, xmlStyle);
          }
-         else if( (action == "Z") || (action == "name") ) {   // Name based on naming template
+         else if(action == "name") {   // Name based on naming template
             size_t p;
             size_t begin      = 0;
             size_t len        = strlen(namingTemplate);
@@ -645,10 +645,10 @@ std::string PublicationSet::applyTemplate(Node*                           public
                result += string2utf8(&namingTemplate[begin], nbsp, lineBreak, xmlStyle);
             }
          }
-         else if( (action == "#") || (action == "download-file-name") ) {   // Download file name
+         else if(action == "download-file-name") {   // Download file name
             child = findChildNode(publication, "url.mime");
             result += makeDownloadFileName(downloadDirectory, publication->keyword,
-                                           (child != NULL) ? child->value : "");
+                                           (child != nullptr) ? child->value : "");
          }
          else if( (action == "a") || (action == "begin-author-loop") ) {   // Author LOOP BEGIN
             if(authorBegin != std::string::npos) {
@@ -659,7 +659,7 @@ std::string PublicationSet::applyTemplate(Node*                           public
             authorIndex = 0;
             authorBegin = i;
          }
-         else if( (action == "g") || (action == "author-initials") ) {   // Current author given name initials
+         else if(action == "author-initials") {   // Current author given name initials
             if(author) {
                std::string initials   = author->arguments[authorIndex + 2];
                removeBrackets(initials);
@@ -671,7 +671,7 @@ std::string PublicationSet::applyTemplate(Node*                           public
                }
             }
          }
-         else if( (action == "G") || (action == "author-give-name") ) {   // Current author given name
+         else if(action == "author-give-name") {   // Current author given name
             if(author) {
                std::string givenName  = author->arguments[authorIndex + 1];
                removeBrackets(givenName);
@@ -683,7 +683,7 @@ std::string PublicationSet::applyTemplate(Node*                           public
                }
             }
          }
-         else if( (action == "F") || (action == "author-family-name") ) {   // Current author family name
+         else if(action == "author-family-name") {   // Current author family name
             if(author) {
                std::string familyName = author->arguments[authorIndex + 0];
                removeBrackets(familyName);
@@ -711,92 +711,97 @@ std::string PublicationSet::applyTemplate(Node*                           public
                   const std::string comparisonString = action.substr(7);
                   skip = ! (writtenString != comparisonString);
                }
-               else if(action.substr(0, 13) < "is-less-than?") {
+               else if(action.substr(0, 13) == "is-less-than?") {
                   const std::string comparisonString = action.substr(13);
-                  skip = ! (writtenString == comparisonString);
+                  skip = ! (writtenString < comparisonString);
                }
-               else if(action.substr(0, 22) <= "is-less-than-or-equal?") {
+               else if(action.substr(0, 22) == "is-less-than-or-equal?") {
                   const std::string comparisonString = action.substr(22);
-                  skip = ! (writtenString == comparisonString);
+                  skip = ! (writtenString <= comparisonString);
                }
-               else if(action.substr(0, 16) > "is-greater-than?") {
+               else if(action.substr(0, 16) == "is-greater-than?") {
                   const std::string comparisonString = action.substr(16);
-                  skip = ! (writtenString == comparisonString);
+                  skip = ! (writtenString > comparisonString);
                }
-               else if(action.substr(0, 25) >= "is-greater-than-or-equal?") {
+               else if(action.substr(0, 25) == "is-greater-than-or-equal?") {
                   const std::string comparisonString = action.substr(25);
-                  skip = ! (writtenString == comparisonString);
+                  skip = ! (writtenString >= comparisonString);
                }
 
                result.erase(entry.pos);   // Remove the written "test" string.
             }
          }
-         else if( (action == "f") || (action == "is-first-author?") ) {       // IS first author
+         else if(action == "is-first-author?") {       // IS first author
             if(skip == false) {
                skip = ! (authorIndex == 0);
             }
          }
-         else if( (action == "n") || (action == "is-not-first-author?") ) {   // IS not first author
+         else if(action == "is-not-first-author?") {   // IS NOT first author
             if(skip == false) {
-               skip = ! ((author != NULL) && (authorIndex > 0));
+               skip = ! ((author != nullptr) && (authorIndex > 0));
             }
          }
-         else if( (action == "l") || (action == "is-last-author?") ) {        // IS last author
+         else if(action == "is-last-author?") {        // IS last author
             if(skip == false) {
-               skip = ! ((author != NULL) && (authorIndex + 3 >= author->arguments.size()));
+               skip = ! ((author != nullptr) && (authorIndex + 3 >= author->arguments.size()));
             }
          }
-         else if( (action == "A") || (action == "end-author-loop") ) {   // Author LOOP EBD
+         else if(action == "is-not-last-author?") {    // IS NOT last author
+            if(skip == false) {
+               skip = ((author != nullptr) && (authorIndex + 3 >= author->arguments.size()));
+            }
+         }
+         else if(action == "end-author-loop") {   // Author LOOP EBD
             if(authorBegin == std::string::npos) {
                fputs("ERROR: Unexpected author loop end %A -> %a author loop begin needed first!\n", stderr);
                exit(1);
             }
             authorIndex += 3;
-            if( (author != NULL) && (authorIndex < author->arguments.size()) ) {
+            if( (author != nullptr) && (authorIndex < author->arguments.size()) ) {
                i = authorBegin;
             }
             else {
-               author      = NULL;
+               author      = nullptr;
                authorIndex = 0;
             }
          }
-         else if( (action == "T") || (action == "title") ) {   // Title
+         else if(action == "title") {   // Title
             child = findChildNode(publication, "title");
             if(child) { result += string2utf8(child->value, nbsp, lineBreak, xmlStyle); } else { skip = true; }
          }
-         else if( (action == "H") || (action == "how-published") ) {   // HowPublished
+         else if(action == "how-published") {   // HowPublished
             child = findChildNode(publication, "howpublished");
             if(child) { result += string2utf8(child->value, nbsp, lineBreak, xmlStyle); } else { skip = true; }
          }
-         else if( (action == "B") || (action == "booktitle") ) {   // Booktitle
+         else if(action == "booktitle") {   // Booktitle
             child = findChildNode(publication, "booktitle");
             if(child) { result += string2utf8(child->value, nbsp, lineBreak, xmlStyle); } else { skip = true; }
          }
-         else if( (action == "r") || (action == "series") ) {   // Series
+         else if(action == "series") {   // Series
             child = findChildNode(publication, "series");
             if(child) { result += string2utf8(child->value, nbsp, lineBreak, xmlStyle); } else { skip = true; }
          }
-         else if( (action == "J") || (action == "journal") ) {   // Journal
+         else if(action == "journal") {   // Journal
             child = findChildNode(publication, "journal");
             if(child) { result += string2utf8(child->value, nbsp, lineBreak, xmlStyle); } else { skip = true; }
          }
-         else if( (action == "E") || (action == "edition") ) {   // Edition
+         else if(action == "edition") {   // Edition
             child = findChildNode(publication, "edition");
             if(child) { result += string2utf8(child->value, nbsp, lineBreak, xmlStyle); } else { skip = true; }
          }
-         else if( (action == "V") || (action == "volume") ) {   // Volume
+         else if(action == "volume") {   // Volume
             child = findChildNode(publication, "volume");
             if(child) { result += string2utf8(child->value, nbsp, lineBreak, xmlStyle); } else { skip = true; }
          }
-         else if( (action == "t") || (action == "type") ) {   // Type
+         else if(action == "type") {   // Type
             child = findChildNode(publication, "type");
             if(child) { result += string2utf8(child->value, nbsp, lineBreak, xmlStyle); } else { skip = true; }
          }
-         else if( (action == "N") || (action == "number") ) {   // Number
+         else if(action == "number") {   // Number
             child = findChildNode(publication, "number");
             if(child) { result += string2utf8(child->value, nbsp, lineBreak, xmlStyle); } else { skip = true; }
          }
-         else if( (action == "P") || (action == "pages") ) {   // Pages
+         else if(action == "pages") {   // Pages
             child = findChildNode(publication, "pages");
             if(child) { result += string2utf8(child->value, nbsp, lineBreak, xmlStyle); } else { skip = true; }
          }
@@ -804,15 +809,15 @@ std::string PublicationSet::applyTemplate(Node*                           public
             child = findChildNode(publication, "numpages");
             if(child) { result += string2utf8(child->value, nbsp, lineBreak, xmlStyle); } else { skip = true; }
          }
-         else if( (action == "@") || (action == "address") ) {   // Address
+         else if(action == "address") {   // Address
             child = findChildNode(publication, "address");
             if(child) { result += string2utf8(child->value, nbsp, lineBreak, xmlStyle); } else { skip = true; }
          }
-         else if( (action == "Y") || (action == "year") ) {   // Year
+         else if(action == "year") {   // Year
             child = findChildNode(publication, "year");
             if(child) { result += string2utf8(child->value, nbsp, lineBreak, xmlStyle); } else { skip = true; }
          }
-         else if( (action == "M") || (action == "month-name") ) {   // Month as name
+         else if(action == "month-name") {   // Month as name
             child = findChildNode(publication, "month");
             if(child) {
                if( (child->number >= 1) && (child->number <= 12) ) {
@@ -820,7 +825,7 @@ std::string PublicationSet::applyTemplate(Node*                           public
                } else { skip = true; }
             } else { skip = true; }
          }
-         else if( (action == "m") || (action == "month-number") ) {   // Month as number
+         else if(action == "month-number") {   // Month as number
             child = findChildNode(publication, "month");
             if(child) {
                char month[16];
@@ -828,7 +833,7 @@ std::string PublicationSet::applyTemplate(Node*                           public
                result += string2utf8(month, nbsp, lineBreak, xmlStyle);
             } else { skip = true; }
          }
-         else if( (action == "D") || (action == "day") ) {   // Day
+         else if(action == "day") {   // Day
             child = findChildNode(publication, "day");
             if(child) {
                char day[16];
@@ -840,51 +845,51 @@ std::string PublicationSet::applyTemplate(Node*                           public
             child = findChildNode(publication, "publisher");
             if(child) { result += string2utf8(child->value, nbsp, lineBreak, xmlStyle); } else { skip = true; }
          }
-         else if( (action == "S") || (action == "school") ) {   // School
+         else if(action == "school") {   // School
             child = findChildNode(publication, "school");
             if(child) { result += string2utf8(child->value, nbsp, lineBreak, xmlStyle); } else { skip = true; }
          }
-         else if( (action == "?") || (action == "institution") ) {   // Institution
+         else if(action == "institution") {   // Institution
             child = findChildNode(publication, "institution");
             if(child) { result += string2utf8(child->value, nbsp, lineBreak, xmlStyle); } else { skip = true; }
          }
-         else if( (action == "I") || (action == "isbn") ) {   // ISBN
+         else if(action == "isbn") {   // ISBN
             child = findChildNode(publication, "isbn");
             if(child) { result += string2utf8("ISBN~" + child->value, nbsp, lineBreak, xmlStyle); } else { skip = true; }
          }
-         else if( (action == "i") || (action == "issn") ) {   // ISSN
+         else if(action == "issn") {   // ISSN
             child = findChildNode(publication, "issn");
             if(child) { result += string2utf8("ISSN~" + child->value, nbsp, lineBreak, xmlStyle); } else { skip = true; }
          }
-         else if( (action == "x") || (action == "language") ) {   // Language
+         else if(action == "language") {   // Language
             child = findChildNode(publication, "language");
             if(child) { result += string2utf8(child->value, nbsp, lineBreak, xmlStyle); } else { skip = true; }
          }
-         else if( (action == "O") || (action == "content-language") ) {   // Content Language
+         else if(action == "content-language") {   // Content Language
             child = findChildNode(publication, "content-language");
-            if(child == NULL) {   // No content language -> try same as "language" instead:
+            if(child == nullptr) {   // No content language -> try same as "language" instead:
                child = findChildNode(publication, "language");
             }
             if(child) { result += string2utf8(child->value, nbsp, lineBreak, xmlStyle); } else { skip = true; }
          }
-         else if( (action == "x") || (action == "xml-language") ) {   // Language
+         else if(action == "xml-language") {   // Language
             child = findChildNode(publication, "language");
             if(child) {
                const char* language = getXMLLanguageFromLaTeX(child->value.c_str());
-               if(language != NULL) {
+               if(language != nullptr) {
                   result += std::string(language);
                } else { skip = true; }
             } else { skip = true; }
          }
-         else if( (action == "U") || (action == "url") ) {   // URL
+         else if(action == "url") {   // URL
             child = findChildNode(publication, "url");
             if(child) { result += string2utf8(child->value, "", "", xmlStyle); } else { skip = true; }
          }
-         else if( (action == "d") || (action == "doi") ) {   // DOI
+         else if(action == "doi") {   // DOI
             child = findChildNode(publication, "doi");
             if(child) { result += string2utf8(child->value, "", "", xmlStyle); } else { skip = true; }
          }
-         else if( (action == "q") || (action == "urn") ) {   // URN
+         else if(action == "urn") {   // URN
             child = findChildNode(publication, "urn");
             if(child) { result += string2utf8(child->value, "", "", xmlStyle); } else { skip = true; }
          }
@@ -900,11 +905,11 @@ std::string PublicationSet::applyTemplate(Node*                           public
             child = findChildNode(publication, "url.md5");
             if(child) { result += string2utf8(child->value, nbsp, lineBreak, xmlStyle); } else { skip = true; }
          }
-         else if( (action == "z") || (action == "url-mime") ) {   // URL mime type
+         else if(action == "url-mime") {   // URL mime type
             child = findChildNode(publication, "url.mime");
             if(child) { result += string2utf8(child->value, "", "", xmlStyle); } else { skip = true; }
          }
-         else if( (action == "y") || (action == "url-type") ) {   // URL type
+         else if(action == "url-type") {   // URL type
             child = findChildNode(publication, "url.mime");
             if(child) {
                if(child->value == "application/pdf") {
@@ -927,7 +932,7 @@ std::string PublicationSet::applyTemplate(Node*                           public
                }
             } else { skip = true; }
          }
-         else if( (action == "s") || (hasPrefix(action, "url-size-",  type)) ) {   // URL size
+         else if(hasPrefix(action, "url-size-",  type)) {   // URL size
             if((action.size() == 1) && (i + 2 < printingTemplateSize)) {
                switch(printingTemplate[i + 2]) {
                   case 'K':   // KiB
@@ -958,7 +963,7 @@ std::string PublicationSet::applyTemplate(Node*                           public
             }
             else { skip = true; }
          }
-         else if( (action == "X") || (action == "note") ) {   // Note
+         else if(action == "note") {   // Note
             child = findChildNode(publication, "note");
             if(child) {
                if( (strncmp(child->value.c_str(), "ISBN", 4) == 0) ||
@@ -975,9 +980,9 @@ std::string PublicationSet::applyTemplate(Node*                           public
          else if(action == "%") {   // %
             result += '%';
          }
-         else if( (action == "b") || (hasPrefix(action, "begin-subdivision-",  type)) ||
-                  (action == "w") || (hasPrefix(action, "within-subdivision-", type)) ||
-                  (action == "e") || (hasPrefix(action, "end-subdivision-",    type)) ) {   // Begin/Within/End of subdivision
+         else if( (hasPrefix(action, "begin-subdivision-",  type)) ||
+                  (hasPrefix(action, "within-subdivision-", type)) ||
+                  (hasPrefix(action, "end-subdivision-",    type)) ) {   // Begin/Within/End of subdivision
             if(i + 2 < printingTemplateSize) {
                if ((action.size() == 1) && (i + 2 < printingTemplateSize) ) {
                   switch(printingTemplate[i + 2]) {
@@ -1005,14 +1010,14 @@ std::string PublicationSet::applyTemplate(Node*                           public
                      exit(1);
                   }
                }
-               const Node* prevChild = (prevPublication != NULL) ? findChildNode(prevPublication, type.c_str()) : NULL;
+               const Node* prevChild = (prevPublication != nullptr) ? findChildNode(prevPublication, type.c_str()) : nullptr;
                child                 = findChildNode(publication, type.c_str());
-               const Node* nextChild = (nextPublication != NULL) ? findChildNode(nextPublication, type.c_str()) : NULL;
+               const Node* nextChild = (nextPublication != nullptr) ? findChildNode(nextPublication, type.c_str()) : nullptr;
 
-               bool begin = (prevChild == NULL) ||
-                           ( (prevChild != NULL) && (child != NULL) && (prevChild->value != child->value) );
-               bool end = (nextChild == NULL) ||
-                           ( (child != NULL) && (nextChild != NULL) && (child->value != nextChild->value) );
+               bool begin = (prevChild == nullptr) ||
+                           ( (prevChild != nullptr) && (child != nullptr) && (prevChild->value != child->value) );
+               bool end = (nextChild == nullptr) ||
+                           ( (child != nullptr) && (nextChild != nullptr) && (child->value != nextChild->value) );
                switch(action[0]) {
                   case 'b':
                      skip = ! begin;
@@ -1026,15 +1031,15 @@ std::string PublicationSet::applyTemplate(Node*                           public
                }
             }
          }
-         else if( (action == "1") || (action == "custom-1") ||
-                  (action == "2") || (action == "custom-2") ||
-                  (action == "3") || (action == "custom-3") ||
-                  (action == "4") || (action == "custom-4") ||
-                  (action == "5") || (action == "custom-5") ||
-                  (action == "6") || (action == "custom-6") ||
-                  (action == "7") || (action == "custom-7") ||
-                  (action == "8") || (action == "custom-8") ||
-                  (action == "9") || (action == "custom-9") ) {   // Custom #1..9
+         else if( (action == "custom-1") ||
+                  (action == "custom-2") ||
+                  (action == "custom-3") ||
+                  (action == "custom-4") ||
+                  (action == "custom-5") ||
+                  (action == "custom-6") ||
+                  (action == "custom-7") ||
+                  (action == "custom-8") ||
+                  (action == "custom-9") ) {   // Custom #1..9
             const unsigned int id = action[action.size() - 1] - '1';
             if(publication->custom[id] != "") {
                result += string2utf8(publication->custom[id], nbsp, lineBreak, xmlStyle);
@@ -1060,6 +1065,25 @@ std::string PublicationSet::applyTemplate(Node*                           public
                skip = true;
             }
          }
+         else if(action.substr(0, 4) == "map:") {   // Map from mappings
+            const std::string mappingName(action.substr(4));
+            const MappingEntry* mappingEntry = mappings.findMapping(mappingName);
+            if(mappingEntry == nullptr) {
+               fprintf(stderr, "ERROR: Mapping \"%s\" does not exist! Forgot parameter \"--mapping %s:mapping_file:key_column:value_column\"?\n", mappingName.c_str(), mappingName.c_str());
+               exit(1);
+            }
+            StackEntry  entry = stack.back();
+            std::string value;
+            std::string key  = result.substr(entry.pos);
+            // fprintf(stderr, "<%s> => ", key.c_str());
+            replaceAll(key, nbsp, " ");
+            // fprintf(stderr, "<%s>\n", key.c_str());
+            skip = !mappings.map(mappingEntry, key, value);
+            if(!skip) {
+               result.erase(entry.pos);   // Remove the written key string.
+               result += value;
+            }
+         }
          else if(action == "exec") {   // Execute command and pipe in the result
             if(i + 1 < printingTemplateSize) {
                StackEntry        entry = stack.back();
@@ -1070,7 +1094,7 @@ std::string PublicationSet::applyTemplate(Node*                           public
                   result.erase(entry.pos);   // Remove the written "exec" string.
 
                   FILE* pipe = popen(call.c_str(), "r");
-                  if(pipe == NULL) {
+                  if(pipe == nullptr) {
                      fprintf(stderr, "Unable to run %s!\n", call.c_str());
                      exit(1);
                   }
@@ -1119,20 +1143,24 @@ std::string PublicationSet::applyTemplate(Node*                           public
          }
          i++;
       }
-      else if(printingTemplate[i] == '[') {
+      else if( (printingTemplate[i] == '[') ||
+               (printingTemplate[i] == '(') ) {
          if(stack.empty()) {
             skip = false;   // Up to now, everything will be accepted
          }
          struct StackEntry entry = { result.size(), skip };
          stack.push_back(entry);
       }
-      else if(printingTemplate[i] == ']') {
+      else if( (printingTemplate[i] == ']') ||
+               (printingTemplate[i] == ')') ) {
          if(!stack.empty()) {
             StackEntry entry = stack.back();
             stack.pop_back();
             if(skip == true) {
                result.erase(entry.pos);
-               skip = entry.skip;
+               if(printingTemplate[i] == ']') {
+                  skip = entry.skip;
+               }
             }
          }
          else {
@@ -1229,9 +1257,10 @@ bool PublicationSet::exportPublicationSetToCustom(PublicationSet*
                                                   const std::string&              lineBreak,
                                                   const bool                      xmlStyle,
                                                   const char*                     downloadDirectory,
+                                                  const Mappings&                 mappings,
                                                   FILE*                           fh)
 {
-   Node* publication = NULL;
+   Node* publication = nullptr;
    gNumber           = 0;
    for(size_t index = 0; index < publicationSet->size(); index++) {
       // ====== Get prev, current and next publications =====================
@@ -1241,10 +1270,10 @@ bool PublicationSet::exportPublicationSetToCustom(PublicationSet*
       Node* prevPublication = publication;
       publication = publicationSet->get(index);
       size_t nextPublicationIndex = 1;
-      Node* nextPublication = (index + nextPublicationIndex< publicationSet->size()) ? publicationSet->get(index + nextPublicationIndex) : NULL;
-      while( (nextPublication != NULL) && (nextPublication->value == "Comment")) {
+      Node* nextPublication = (index + nextPublicationIndex< publicationSet->size()) ? publicationSet->get(index + nextPublicationIndex) : nullptr;
+      while( (nextPublication != nullptr) && (nextPublication->value == "Comment")) {
          nextPublicationIndex++;
-         nextPublication = (index + nextPublicationIndex< publicationSet->size()) ? publicationSet->get(index + nextPublicationIndex) : NULL;
+         nextPublication = (index + nextPublicationIndex< publicationSet->size()) ? publicationSet->get(index + nextPublicationIndex) : nullptr;
       }
 
       const std::string result = applyTemplate(publication, prevPublication, nextPublication,
@@ -1252,6 +1281,7 @@ bool PublicationSet::exportPublicationSetToCustom(PublicationSet*
                                                printingTemplate,
                                                monthNames, nbsp, lineBreak, xmlStyle,
                                                downloadDirectory,
+                                               mappings,
                                                fh);
 
       fputs(string2utf8(processBackslash(customPrintingHeader), nbsp).c_str(), stdout);
